#!/bin/bash

# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

Save_result() {
    _RESULT_TYPE="${1}"
    _IMAGE_TYPE="${2}"
    _FILE_PATH="${3}"
    _DONT_RENAME="${4}"

    Debug "Save_result called with result type ${_RESULT_TYPE} and image type '${_IMAGE_TYPE}' for file '${_FILE_PATH}'."
    if [ -z "${_RESULT_TYPE}" ] || [ -z "${_IMAGE_TYPE}" ] || [ -z "${_FILE_PATH}" ]
    then
        Error "Save_result requires three arguments: RESULT_TYPE IMAGE_TYPE FILE_PATH"
        exit 1
    fi

    if [ "${_RESULT_TYPE}" = "binary" ]
    then
        _DESTINATION="${RESULTDIR_IMAGES}/${_IMAGE_TYPE}"
    elif [ "${_RESULT_TYPE}" = "source" ]
    then
        _DESTINATION="${RESULTDIR_SOURCES}/${_IMAGE_TYPE}"
    else
        Error "Invalid result type passed to Save_result; RESULT_TYPE should be 'binary' or 'source'."
        exit 1
    fi

    Debug "Ensuring destination directory '${_DESTINATION}' exists."
    mkdir -p ${_DESTINATION}

    if [ -z "${_DONT_RENAME}" ]; then
        _NEW_FILENAME="$(basename ${_FILE_PATH} | sed s/$(basename ${_FILE_PATH} | cut -d'.' -f1)/${PROJECT}-${_IMAGE_TYPE}-${BUILDNAME}/)"
    else
        _NEW_FILENAME="$(basename ${_FILE_PATH})"
    fi

    unset IFS
    for _CHECKSUM_TYPE in ${IBS_CHECKSUM_TYPES}
    do
        Debug "Generating ${_CHECKSUM_TYPE} message digest for file ${_NEW_FILENAME} (${_FILE_PATH})"
        _FILE_HASH="$(Calculate_hash ${_CHECKSUM_TYPE} ${_FILE_PATH})"
        echo "${_FILE_HASH}  ${_NEW_FILENAME}" >> "${_DESTINATION}/${PROJECT}-${BUILDNAME}.${_CHECKSUM_TYPE}sums.txt"
    done
    
    Debug "Copying file '${_FILE_PATH}' to '${_DESTINATION}/${_NEW_FILENAME}'"
    cp "${_FILE_PATH}" "${_DESTINATION}/${_NEW_FILENAME}"

    Debug "Verifying integrity of copy"
    for _CHECKSUM_TYPE in ${IBS_CHECKSUM_TYPES}
    do
        _COPY_HASH="$(Calculate_hash ${_CHECKSUM_TYPE} "${_DESTINATION}/${_NEW_FILENAME}")"
        _ORIGINAL_HASH="$(grep "${_NEW_FILENAME}$" "${_DESTINATION}/${PROJECT}-${BUILDNAME}.${_CHECKSUM_TYPE}sums.txt" | awk '{ print $1 }' )"
        if [ "${_COPY_HASH}" != "${_ORIGINAL_HASH}" ]
        then
            Error "Integrity check of ${_DESTINATION}/${_NEW_FILENAME} failed."
            Print "\tExpected ${_CHECKSUM_TYPE} hash: %-20s Actual ${_CHECKSUM_TYPE} Hash: %s" "${_ORIGINAL_HASH}" "${_COPY_HASH}"
            Error "Copying of build results failed on integrity test."
            exit 1
        else
            Debug "Verification was successful (%s == %s)" "${_ORIGINAL_HASH}" "${_COPY_HASH}"
        fi
    done
}

Calculate_hash() {
    _HASH_TYPE="${1}"
    _FILE_PATH="${2}"

    Debug "Calculate_hash called with hash type '${_HASH_TYPE}' for file '${_FILE_PATH}'."
    if [ -z "${_HASH_TYPE}" ] || [ -z "${_FILE_PATH}" ]
    then
        Error "Calculate_hash requires two arguments: HASH_TYPE FILE_PATH"
        exit 1
    fi

    case "${_HASH_TYPE}" in
        md5)
            echo "$(md5sum ${_FILE_PATH} | cut -d' ' -f1)"
            ;;

        sha1)
            echo "$(sha1sum ${_FILE_PATH} | cut -d' ' -f1)"
            ;;

        sha224)
            echo "$(sha224sum ${_FILE_PATH} | cut -d' ' -f1)"
            ;;

        sha256)
            echo "$(sha256sum ${_FILE_PATH} | cut -d' ' -f1)"
            ;;

        sha384)
            echo "$(sha384sum ${_FILE_PATH} | cut -d' ' -f1)"
            ;;

        sha512)
            echo "$(sha512sum ${_FILE_PATH} | cut -d' ' -f1)"
            ;;

        *)
            Error "Unsupported hash type '${_HASH_TYPE}' requested for '${_FILE_PATH}'."
            exit 1
            ;;
    esac
}
