#!/bin/bash

# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

Buildtool_setup() {
    Catch_faults permit_errors

    TARGETFS="${IBS_WORKDIR}/${PROJECT}/chroot"

    sudo apt-get update -o Acquire::http::No-Cache=True
    ERROR=$?
    if [ "${ERROR}" != "0" ]; then
        Error "Error running apt-get update in preperation of getting latest live-helper"
        Die
    fi

    # Default to installing live-helper 1.x
    IBS_BUILDTOOL_VERSION="${IBS_BUILDTOOL_VERSION:-1.*}"
    sudo apt-get install -y --force-yes live-helper="${IBS_BUILDTOOL_VERSION}"
    ERROR=$?
    if [ "${ERROR}" != "0" ]; then
        Error "Error installing live-helper=${IBS_BUILDTOOL_VERSION}"
        Die
    fi

    if [ -d "${IBS_WORKDIR}/${PROJECT}/config" ]; then
        rm -rf "${IBS_WORKDIR}/${PROJECT}/config" > /dev/null 2>&1 || true
    fi

    Action "Copying project configuration directory into work directory for build"
    Export_project_config "${PROJECT}" "${IBS_WORKDIR}/${PROJECT}/config"
    Action_done

    if [ "$?" != "0" ]; then
        Error "Unable to copy project build configuration to %s" "${IBS_WORKDIR}/${PROJECT}/config"
        Die
    fi

    Action "Generating buildstamp"
    mkdir -p "${IBS_WORKDIR}"/"${PROJECT}"/config/chroot_local-includes/etc/
    cat >> "${IBS_WORKDIR}"/"${PROJECT}"/config/chroot_local-includes/etc/buildstamp << EOF
${HOSTNAME} $(date -R)
${PROJECT}-${BUILDNAME}
EOF
    Action_done

    if [ ! -z "${IBS_DISTRIBUTION_CHANNEL_DESCRIPTOR_PREFIX}" ]
    then
        Action "Generating distribution channel descriptor"
        mkdir -p "${IBS_WORKDIR}"/"${PROJECT}"/config/chroot_local-includes/var/lib/
        cat > "${IBS_WORKDIR}"/"${PROJECT}"/config/chroot_local-includes/var/lib/ubuntu_dist_channel << EOF
# This is a distribution channel descriptor
# For more information see http://wiki.ubuntu.com/DistributionChannelDescriptor
${IBS_DISTRIBUTION_CHANNEL_DESCRIPTOR_PREFIX}-${PROJECT}-${BUILDNAME}
EOF
        Action_done
    else
        Info "Skipping generation of distribution channel descriptor"
    fi

    Catch_faults disallow_errors

    Load_script_configfile "${IBS_WORKDIR}/${PROJECT}/config/bootstrap"
    Load_script_configfile "${IBS_WORKDIR}/${PROJECT}/config/chroot"
    Load_script_configfile "${IBS_WORKDIR}/${PROJECT}/config/binary"
    Load_script_configfile "${IBS_WORKDIR}/${PROJECT}/config/source"
}

Buildtool_build() {
    _Check_directory

    sudo lh build

    ERROR=$?
    if [ "${ERROR}" != "0" ]; then
        Error "Couldn't create ${LH_BINARY_IMAGES} image"
        Die
    fi

    _SaveConfig

    _DESTINATION_DIRECTORY="${RESULTDIR_IMAGES}/${LH_BINARY_IMAGES}/"
    _RESULT_BASENAME="${PROJECT}-${LH_BINARY_IMAGES}-${BUILDNAME}"
    mkdir -p "${_DESTINATION_DIRECTORY}"

    Action "Generating checksums and copying build product to results directory"
    IFS="$(echo -e '\n\r')"
    Debug "Processing binary image results"
    for FILENAME in $(find . -maxdepth 1 -type f -name "binary*"); do
        Save_result "binary" "${LH_BINARY_IMAGES}" "${FILENAME}"
    done
    Debug "Processing source image results"
    for FILENAME in $(find . -maxdepth 1 -type f -name "source*"); do
        Save_result "source" "${LH_SOURCE_IMAGES}" "${FILENAME}"
    done
    Action_done
    unset IFS

    produceBuildComparison "${LH_BINARY_IMAGES}"

    if [ "${ARCHIVEPROJECT}" = "1" ] || [ "${ARCHIVEPROJECT}" = "enabled" ]; then
        Info "Creating archive of build work directory..."
        mkdir -p "${RESULTDIR_PROJDIR}/"
        _SaveBuildWorkDirectory
    fi
}

Buildtool_cleanup() {
    _Check_directory

    sudo lh clean --purge
    rm -rf "${IBS_WORKDIR}/${PROJECT}/config"
}

_Check_directory() {
    if [ "${PWD}" != "${IBS_WORKDIR}/${PROJECT}" ]; then
        Error "Working directory is '%s'; expected '%s'" "{$PWD}" "${IBS_WORKDIR}/${PROJECT}"
        Die
    fi
    Debug "Directory check performed and was successful; PWD: %s" "${PWD}"
}

_SaveConfig() {
    #XXX: Hack so that describe-changes works.
    mkdir -p "${RESULTDIR_CONFIGFILES}/sources-list/"

    for FILE in $(find "${IBS_WORKDIR}/${PROJECT}/config/chroot_sources/" -maxdepth 1 -type f -name "*.chroot" )
    do
        cp "${IBS_WORKDIR}/${PROJECT}/config/chroot_sources/$(basename ${FILE})" "${RESULTDIR_CONFIGFILES}/sources-list/"
    done

    # LH_CATEGORIES was renamed to LH_ARCHIVE_AREAS in live-helper 2.x
    echo "deb ${LH_MIRROR_CHROOT} ${LH_DISTRIBUTION} ${LH_CATEGORIES:-${LH_ARCHIVE_AREAS}}" > "${RESULTDIR_CONFIGFILES}/sources-list/sources.list"
    echo "deb-src ${LH_MIRROR_CHROOT} ${LH_DISTRIBUTION} ${LH_CATEGORIES:-${LH_ARCHIVE_AREAS}}" >> "${RESULTDIR_CONFIGFILES}/sources-list/sources.list"
}

_SaveBuildWorkDirectory() {
    _Check_directory

    Action "Removing generated images before archiving"
    sudo lh clean --binary
    sudo lh clean --source
    Action_done

    mkdir -p "${RESULTDIR_PROJDIR}/"

    Action "Generating build archive (may take awhile)"
    tar -zcf "${RESULTDIR_PROJDIR}/${PROJECT}-${BUILDNAME}.build.tgz" "${IBS_WORKDIR}/${PROJECT}" > /dev/null 2>&1
    Action_done
}
