# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import logging
import time
import sys
import xmlrpclib

from psycopg2.extensions import TransactionRollbackError
import storm
from storm.locals import (
    create_database,
    Desc,
    Store
)

from offspring import config
from offspring.daemon import LexbuilderDaemon
from offspring.master.exceptions import SlaveCommunicationError
from offspring.master.models import (
    BuildRequest,
    Lexbuilder
)
from offspring.master import notifications
from offspring.slave import LexbuilderSlave


class LexbuilderMaster(LexbuilderDaemon):

    def __init__(self):
        self.isOnline = False
        self.setup()
        LexbuilderDaemon.__init__(self)

    def setup(self):
        logging.basicConfig(
            level=logging.INFO,
            format='%(asctime)s %(levelname)-8s %(message)s',
            datefmt='%a, %d %b %Y %H:%M:%S',
            filename=config.master('logfile'),
            filemode='w')
        logging.log(100, "Starting up offspring master")
        try:
            self.database = create_database(config.master("db"))
            self.db_store = Store(self.database)
        except storm.exceptions.OperationalError, e:
            logging.exception("Unable to connect to database.")
            raise e
        self.isOnline = True

    def run(self):
        while self.isOnline:
            try:
                self.db_store.flush()
                self.scanSlaves()
                self.dispatchBuilds()
            except storm.exceptions.DisconnectionError:
                logging.exception("Master was disconnected from database.")
                logging.info("Waiting 20 seconds and attempting reconnect.")
                time.sleep(20)
                self.setup()
            except TransactionRollbackError:
                logging.exception("TransactionRollbackError occurred... ")
                logging.info("Performing rollback to recover from error.")
                self.db_store.rollback()
            except:
                logging.exception(
                    "Uncaught error propagated to main execution loop.")
                notifications.notify_crash(
                    "Lexbuilder Master (Build Queue Dispatcher)")
                logging.error(
                    "SHUTTING DOWN: Fatal error has occurred.")
                sys.exit(1)
            finally:
                time.sleep(int(config.master("poll_frequency")))

    def stop(self):
        self.isOnline = False

    def scanSlaves(self):
        for builder in self.db_store.find(Lexbuilder):
            try:
                oldState, newState = builder.scan()
            except SlaveCommunicationError, issue:
                logging.error(
                    "Scan of %s failed. %s" % (builder.name, issue))
                builder.is_okay = False
                self.db_store.commit()
                continue
            self.db_store.commit()
            logging.debug("%s: %s->%s" % (builder.name, oldState, newState))
            if (builder.current_state == LexbuilderSlave.STATE_IDLE and 
                builder.current_job != None):
                try:
                    if builder.getBuildResult():
                        logging.info("%s has finished building on %s" % 
                            (builder.current_job.project.name, builder.name))
                        #XXX: Notification stuff needs work.
                        try:
                            notifications.notify_buildresult(
                                builder.current_job)
                        except:
                            logging.exception(
                                "Attempt to send build result notification e-mail failed.")
                            logging.error(
                                "PROBLEM DETECTED: A non-fatal problem has been detected. See above for details.")
                            try:
                                notifications.notify_problem(
                                    "Lexbuilder Master (Build Queue Dispatcher)",
                                    "unable to send build result notification")
                            except:
                                logging.exception(
                                    "Attempt to send problem notification e-mail failed.")
                                logging.error(
                                    "PROBLEM DETECTED: A non-fatal problem has been detected. See above for details.")
                        builder.current_job = None
                        self.db_store.commit()
                    else:
                        logging.error(
                            "Failed to gather build result from %s" % 
                            (builder.name))
                        logging.info(
                            "Deleting %s build record %s for lost build on %s" % 
                            (builder.current_job.project.name, 
                             builder.current_job, builder.name))
                        self.db_store.remove(builder.current_job)
                        builder.current_job = None
                        self.db_store.commit()
                except SlaveCommunicationError, issue:
                    logging.error(
                        "Failed to gather build results from %s. %s" % 
                        (builder.name, issue))
                    builder.is_okay = False
                    self.db_store.commit()
                    continue
            self.db_store.flush()
        logging.debug("Scan cycle completed")

    def dispatchBuilds(self):
        #XXX: ARCH_MACHINE_SUPPORT is a bit of hack
        ARCH_MACHINE_SUPPORT = { 
                u'i386'  : [u'i386', u'i686', u'x86_64'],
                u'lpia'  : [u'i386', u'i686', u'x86_64'],
                u'amd64' : [u'x86_64'],
                u'armel' : [u'armv7l'], 
        }
        for request in self.db_store.find(BuildRequest).order_by(Desc(BuildRequest.score)):
            project = request.project
            logging.debug(
                "Project %s needs building (priority: %s)." % 
                (project.name, project.priority))
            builder = self.db_store.find(Lexbuilder, 
                    (Lexbuilder.current_state == LexbuilderSlave.STATE_IDLE) & 
                    (Lexbuilder.is_active == True) &
                    (Lexbuilder.is_okay == True) & 
                    (Lexbuilder.machine_type.is_in(
                        ARCH_MACHINE_SUPPORT[unicode(project.arch)]))
                ).order_by(Lexbuilder.id).first()
            if builder is not None:
                try:
                    job = builder.build(request)
                    if job != None:
                        self.db_store.add(job)
                        self.db_store.remove(request)
                        self.db_store.commit()
                        logging.info(
                            "%s has been dispatched to %s" % 
                            (project.name, builder.name))
                    else:
                        logging.error(
                            "Error when asking %s to build %s" % 
                            (builder.name, project.name))
                except SlaveCommunicationError, issue:
                    logging.error(
                        "Failed to dispatch %s build to %s. %s" % 
                        (project.name, builder.name, issue))
                    builder.is_okay = False
                    self.db_store.commit()
                    continue
            else:
                logging.debug(
                    "Unable to find builder to build %s" % project.name)
        self.db_store.flush()
