# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__all__ = ( 'Lexbuilder', 'Project', 'BuildResult', 'BuildRequest', 'DailyBuildOrder', 
    'LaunchpadProject', 'LaunchpadProjectMilestone', 'ProjectNotificationSubscription', 
    'Release' )

import datetime
import socket
import xmlrpclib

from storm.locals import (
    Bool,
    Date,
    DateTime,
    Int,
    Reference,
    ReferenceSet,
    Store,
    Storm,
    Unicode
)
from storm import tz

from offspring import config
from offspring.master.exceptions import SlaveCommunicationError
from offspring.networking import TimeoutTransport
from offspring.slave import LexbuilderSlave, LexbuilderSlaveServer
from offspring.enums import ProjectBuildStates


class Lexbuilder(Storm):
    __storm_table__ = "lexbuilders"
    id = Int(primary=True)
    name = Unicode(default=u"UNKNOWN")
    uri = Unicode()
    machine_type = Unicode(default=u"x86_64")
    current_state = Unicode(default=LexbuilderSlave.STATE_UNKNOWN)
    previous_state = Unicode()
    created_at = DateTime(default=datetime.datetime.now(tz.tzlocal()))
    updated_at = DateTime()
    is_active = Bool(default=True)
    is_okay = Bool(default=True)
    current_job_id = Int()
    current_job = Reference(current_job_id, "BuildResult.id")
    notes = Unicode()
    
    class ServerProxyWrapper(object):
        
        def __init__(self, server_proxy):
            self.__server_proxy = server_proxy
        
        def __getattr__(self, name):
            attr = getattr(self.__server_proxy, name)
            if hasattr(attr, '__call__'):
                def wrapped_call(*args, **kwargs):
                    try:
                        return attr(*args, **kwargs)
                    except socket.error, socket_error:
                        raise SlaveCommunicationError(
                            "Socket error: %s" % socket_error)
                    except xmlrpclib.ProtocolError, protocol_error:
                        raise SlaveCommunicationError(
                            "Protocol error: %s" % protocol_error.errmsg)
                    except xmlrpclib.Fault, slave_fault:
                        raise SlaveCommunicationError(
                            "Fault occurred: %s" % slave_fault.faultString)
                return wrapped_call
            else:
                return attr
    
    def __init__(self, uri):
        self.uri = unicode(uri)
        self.server_proxy = self.connect()
    
    def __str__(self):
        return self.name
    
    def __storm_loaded__(self):
        self.server_proxy = self.connect()
    
    def connect(self):
        server_proxy = Lexbuilder.ServerProxyWrapper(
            xmlrpclib.ServerProxy(
                self.uri, transport=TimeoutTransport(), allow_none=True))
        return server_proxy
    
    def scan(self):
        self.name = unicode(self.server_proxy.getBuilderName())
        self.machine_type = unicode(self.server_proxy.getMachineType())
        self.previous_state = self.current_state
        self.current_state = unicode(self.server_proxy.getBuilderState())
        self.updated_at = datetime.datetime.now(tz.tzlocal())
        self.is_okay = True
        
        if self.current_state == LexbuilderSlave.STATE_BUILDING:
            if self.current_job:
                if self.current_job.name == None:
                    buildName = unicode(self.server_proxy.getBuildName())
                    if buildName != LexbuilderSlaveServer.REQUEST_ERROR:
                        self.current_job.name = buildName
                if self.current_job.started_at == None:
                    startedAt = self.server_proxy.getBuildStartedAt()
                    self.current_job.started_at = datetime.datetime.strptime(
                        startedAt.value,
                        "%Y%m%dT%H:%M:%S").replace(tzinfo=tz.tzlocal())
        
        return (self.previous_state, self.current_state)
    
    def build(self, request):
        if self.server_proxy.startBuild(request.project.name, request.project.config_url) == LexbuilderSlaveServer.REQUEST_OKAY:
            self.previous_state = self.current_state
            self.current_state = LexbuilderSlave.STATE_BUILDING
            self.current_job = BuildResult(request, self)
            return self.current_job
        else:
            return None

    def getBuildResult(self):
        #XXX: We should be able to handle when we don't already have a
        #     build result record created like before; required to collect
        #     date from manually started builds.
        if self.current_job != None:
            if self.server_proxy.getBuildResult() != LexbuilderSlaveServer.REQUEST_ERROR:
                if self.current_job.started_at is None:
                    self.current_job.started_at = datetime.datetime.strptime(
                        self.server_proxy.getBuildStartedAt().value, 
                        "%Y%m%dT%H:%M:%S").replace(tzinfo=tz.tzlocal())
                else:
                    pass
                self.current_job.finished_at = datetime.datetime.strptime(
                    self.server_proxy.getBuildFinishedAt().value, 
                    "%Y%m%dT%H:%M:%S").replace(tzinfo=tz.tzlocal())
                self.current_job.name = unicode(self.server_proxy.getBuildName())
                self.current_job.result = unicode(self.server_proxy.getBuildResult())
                return True
            elif self.current_job.name is not None:
                # Issue with builder but we know the build at least started so keep build result record.
                self.current_job.finished_at = datetime.datetime.strptime(
                    xmlrpclib.DateTime(datetime.datetime.now()).value,
                    "%Y%m%dT%H:%M:%S").replace(tzinfo=tz.tzlocal())
                self.current_job.result = unicode(
                    ProjectBuildStates.FAILED)
                if self.current_job.started_at is None:
                    self.current_job.started_at = self.current_job.dispatched_at
                else:
                    pass
                self.is_okay = False
                return True
        return False

class Project(Storm):
    __storm_table__ = "projects"
    name = Unicode(primary=True)
    arch = Unicode(default=u"i386")
    project_group_id = Unicode(default=None)
    title = Unicode()
    priority = Int(default=0)
    is_active = Bool(default=True)
    launchpad_project_id = Unicode()
    launchpad_project = Reference(launchpad_project_id, "LaunchpadProject.name")
    suite = Unicode()
    series = Unicode()
    subscriptions = ReferenceSet(name, "ProjectNotificationSubscription.project_name")
    status = Unicode()
    config_url = Unicode()
    notes = Unicode()
    
    def __init__(self, name, priority=40):
        self.title = unicode(name.capitalize())
        self.name = unicode(name)
        self.priority = priority

    def __str__(self):
        return self.name

class BuildResult(Storm):
    __storm_table__ = "buildresults"
    id = Int(primary=True)
    project_name = Unicode()
    project = Reference(project_name, Project.name)
    name = Unicode(default=None)
    requestor_id = Int()
    requestor = Reference(requestor_id, "User.id")
    reason = Unicode()
    requested_at = DateTime()
    dispatched_at = DateTime()
    builder_id = Int()
    builder = Reference(builder_id, Lexbuilder.id)
    result = Unicode(default=None)
    started_at = DateTime()
    finished_at = DateTime()
    notes = Unicode()
    
    def __init__(self, request, builder):
        self.project = request.project
        self.requestor_id = request.requestor_id
        self.reason = request.reason
        self.requested_at = request.created_at
        self.dispatched_at = datetime.datetime.now(tz.tzlocal())
        self.builder = builder
    
    def result_directory(self, base_dir="/srv/builds"):
        try:
            buildDate, buildId = self.name.rsplit('-', 1)
        except:
            return None
        
        return "%s/%s/%s/%s" % (base_dir, self.project.name, buildDate, buildId)

class BuildRequest(Storm):
    __storm_table__ = "buildrequests"
    id = Int(primary=True)
    project_name = Unicode()
    project = Reference(project_name, Project.name)
    requestor_id = Int(default=None)
    created_at = DateTime(default=datetime.datetime.now(tz.tzlocal()))
    score = Int()
    reason = Unicode()
    
    def __init__(self, project, score=50, reason=u""):
        self.project = project
        self.score = score
        self.reason = reason

class DailyBuildOrderProject(object):
    __storm_table__ = "dailybuildorders_projects"
    __storm_primary__ = "dailybuildorder_id", "project_id"
    dailybuildorder_id = Int()
    project_id = Unicode()

class DailyBuildOrder(object):
    __storm_table__ = "dailybuildorders"
    id = Int(primary=True)
    name = Unicode()
    hour = Int()
    projects = ReferenceSet(id,
                            DailyBuildOrderProject.dailybuildorder_id,
                            DailyBuildOrderProject.project_id,
                            Project.name)
    
    def __init__(self, name, hour):
        self.name = name
        self.hour = hour
    
    def __str__(self):
        return self.name

class LaunchpadProject(Storm):
    __storm_table__ = "launchpad_projects"
    name = Unicode(primary=True)
    title = Unicode(default=None)
    projects = ReferenceSet(name, Project.launchpad_project_id)

    def __str__(self):
        return self.name

class LaunchpadProjectMilestone(Storm):
    __storm_table__ = "launchpad_project_milestones"
    id = Int(primary=True)
    name = Unicode()
    launchpad_project_id = Unicode()
    launchpad_project = Reference(launchpad_project_id, LaunchpadProject.name)
    title = Unicode(default=u"")
    date_targeted = Date(default=None)
    active = Bool(default=True)
    
    def __init__(self, name, title, launchpad_project, date_targeted=None, active=True):
        self.name = unicode(name)
        self.launchpad_project = launchpad_project
        self.date_targeted = date_targeted
        self.active = active
        if title is None:
            self.title = u""
        else:
            self.title = unicode(title)

class User(Storm):
    __storm_table__ = "auth_user"
    id = Int(primary=True)
    username = Unicode()
    first_name = Unicode()
    last_name = Unicode()
    email = Unicode()
    password = Unicode()
    is_staff = Bool()
    is_active = Bool()
    is_superuser = Bool()
    last_login = DateTime()
    date_joined = DateTime()

class ProjectNotificationSubscription(Storm):
    __storm_table__ = "project_notification_subscriptions"
    id = Int(primary=True)
    user_id = Int()
    user = Reference(user_id, User.id)
    project_name = Unicode()
    project = Reference(project_name, Project.name)

class Release(Storm):
    STATUS_PENDING, STATUS_PUBLISHED, STATUS_REMOVED, STATUS_MISSING, STATUS_ERROR = range(5)
    __storm_table__ = "releases"
    build_id = Int(primary=True)
    build = Reference(build_id, BuildResult.id)
    name = Unicode()
    milestone_id = Int()
    milestone = Reference(milestone_id, LaunchpadProjectMilestone.id)
    creator_id = Int()
    creator = Reference(creator_id, User.id)
    created_at = DateTime(default=datetime.datetime.now(tz.tzlocal()))
    updated_at = DateTime()
    published_at = DateTime()
    status = Int()
    checklist_url = Unicode()
    notes = Unicode()
    tag = Unicode()
    
    def result_directory(self, base_dir="/srv/partners"):
        return "%s/%s/images/%s" % (base_dir, self.build.project.name, self.build.name)
