# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import email.parser
import email.utils
import unittest

from offspring import config
from offspring.notifications import (
    notify_crash,
    notify_problem,
)


class StubMailer(object):

    def __init__(self):
        self.mails = []

    def mail(self, sender, rcpt, msg):
        self.mails.append((sender, rcpt, msg))


class Factory(object):

    def __init__(self):
        self.counter = 1

    def getUniqueString(self):
        self.counter += 1
        return 'str%s' % self.counter


class TestCaseWithFactory(unittest.TestCase):

    def setUp(self):
        self.factory = Factory()

    def tearDown(self):
        config._read_config()

    def headerFromMailText(self, msgtext, header):
        msg = email.parser.Parser().parsestr(msgtext)
        return msg[header]


class TestNotifyCrash(TestCaseWithFactory):

    def test_notify_crash_reply_to_from_config(self):
        # notify_crash sets the reply-to of the mail it sends to the value of
        # the 'discussion_email' config entry.
        mailer = StubMailer()
        value = self.factory.getUniqueString()
        config.config.set("DEFAULT", "discussion_email", value)
        notify_crash(self.factory.getUniqueString(), mailer.mail)
        self.assertEqual(
            value, self.headerFromMailText(mailer.mails[0][2], 'reply-to'))

    def test_notify_crash_signs_off_with_instance_name(self):
        # notify_crash signs off with the name of the installation.
        mailer = StubMailer()
        value = self.factory.getUniqueString()
        config.config.set("DEFAULT", "instance_name", value)
        notify_crash(self.factory.getUniqueString(), mailer.mail)
        self.assertTrue(mailer.mails[0][2].strip().endswith(value))

    def test_component_name_in_real_name_of_sender(self):
        # The 'real name' part of the Sender of the mail sent by notify_crash
        # is the 'component' parameter.
        mailer = StubMailer()
        component = self.factory.getUniqueString()
        notify_crash(component, mailer.mail)
        from_header = mailer.mails[0][0]
        sender_real_name, sender_addr = email.utils.parseaddr(from_header)
        self.assertEqual(component, sender_real_name)

    def test_instance_email_in_address_of_sender(self):
        # The 'real name' part of the Sender of the mail sent by notify_crash
        # is the 'component' parameter.
        mailer = StubMailer()
        addr = self.factory.getUniqueString()
        config.config.set("DEFAULT", "instance_email", addr)
        notify_crash(self.factory.getUniqueString(), mailer.mail)
        from_header = mailer.mails[0][0]
        sender_real_name, sender_addr = email.utils.parseaddr(from_header)
        self.assertEqual(addr, sender_addr)


class TestNotifyProblem(TestCaseWithFactory):

    def test_notify_problem_signs_off_with_instance_name(self):
        # notify_problem signs off with the name of the installation.
        mailer = StubMailer()
        value = self.factory.getUniqueString()
        config.config.set("DEFAULT", "instance_name", value)
        notify_problem(
            self.factory.getUniqueString(), self.factory.getUniqueString(),
            mailer.mail)
        self.assertTrue(mailer.mails[0][2].strip().endswith(value))

    def test_component_name_in_real_name_of_sender(self):
        # The 'real name' part of the Sender of the mail sent by
        # notify_problem is the 'component' parameter.
        mailer = StubMailer()
        component = self.factory.getUniqueString()
        notify_problem(component, self.factory.getUniqueString(), mailer.mail)
        from_header = mailer.mails[0][0]
        sender_real_name, sender_addr = email.utils.parseaddr(from_header)
        self.assertEqual(component, sender_real_name)

    def test_instance_email_in_address_of_sender(self):
        # The 'real name' part of the Sender of the mail sent by
        # notify_problem is the 'component' parameter.
        mailer = StubMailer()
        addr = self.factory.getUniqueString()
        config.config.set("DEFAULT", "instance_email", addr)
        notify_problem(
            self.factory.getUniqueString(), self.factory.getUniqueString(),
            mailer.mail)
        from_header = mailer.mails[0][0]
        sender_real_name, sender_addr = email.utils.parseaddr(from_header)
        self.assertEqual(addr, sender_addr)


def test_suite():
    loader = unittest.TestLoader()
    return loader.loadTestsFromName(__name__)
