# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__all__ = (
    'LexbuilderSlave',
    'LexbuilderSlaveServer',
    'ProjectBuildProcess',
)

import logging
import platform
import time
import xmlrpclib

from offspring import config
from offspring.daemon import (
    LexbuilderDaemon,
    LexbuilderXMLRPCServer
)
from offspring.enums import ProjectBuildStates
from offspring.networking import BasicAuthXMLRPCRequestHandler
from offspring.slave.taskmonitor import (
    MonitoredProcess,
    MonitoredThread
)


class LexbuilderSlave(LexbuilderDaemon):

    STATE_IDLE, STATE_BUILDING, STATE_UNKNOWN, STATE_UNAVAIL = (
        u"IDLE",
        u"BUILDING",
        u"UNKNOWN",
        u"UNAVAILABLE"
    )

    def __init__(self, addr):
        self.server = LexbuilderSlaveServer(self, addr)
        self.state = LexbuilderSlave.STATE_IDLE
        self.currentBuild = None
        logging.basicConfig(level=logging.DEBUG,
                    format='%(asctime)s %(levelname)-8s %(message)s',
                    datefmt='%a, %d %b %Y %H:%M:%S',
                    filename=config.slave('logfile'),
                    filemode='w')
        logging.info(
            "Starting up offspring slave server on port %s" % addr[1])
        LexbuilderDaemon.__init__(self)

    def run(self):
        self.server.start()
        logging.info("Slave ready to accept builds")

    def stop(self):
        self.stopBuild()
        self.server.stop()
        self.server.join()

    def startBuild(self, projectName, configBranchURL):
        self.currentBuild = ProjectBuildProcess(
            self, projectName, configBranchURL)
        logging.info("Starting build: %s %s %s" % (
            self.currentBuild.getCommandPath(), projectName, configBranchURL))
        self.currentBuild.start()

    def stopBuild(self):
        logging.info("Killing active build.")
        if self.currentBuild is not None:
            if self.currentBuild.process is not None:
                if self.currentBuild.process.poll() is None:
                    self.currentBuild.process.terminate()
            # Give it 5 seconds before sending kill.
            time.sleep(int(config.slave("child_wait_timeout")))
            self.currentBuild.process.kill()


class LexbuilderSlaveServer(LexbuilderXMLRPCServer):

    def __init__(self, slave, addr,
                 requestHandler=BasicAuthXMLRPCRequestHandler,
                 logRequests=False, allow_none=True, encoding="UTF-8"):
        self.slave = slave
        LexbuilderXMLRPCServer.__init__(
            self, addr, requestHandler, logRequests, allow_none, encoding)

    def api_startBuild(self, projectName, configBranchURL):
        if self.slave.currentBuild is not None:
            if self.slave.currentBuild.state == ProjectBuildProcess.STATE_ACTIVE:
                return LexbuilderSlaveServer.REQUEST_ERROR
        self.slave.startBuild(projectName, configBranchURL)
        return LexbuilderSlaveServer.REQUEST_OKAY

    def api_stopBuild(self):
        if not platform.version() < "2.6" and self.slave.currentBuild is not None:
            if self.slave.currentBuild.state == ProjectBuildProcess.STATE_ACTIVE:
                self.slave.stopBuild()
                return LexbuilderSlaveServer.REQUEST_OKAY
        return LexbuilderSlaveServer.REQUEST_ERROR

    def api_getBuilderName(self):
        return platform.node()

    def api_getMachineType(self):
        return platform.machine()

    def api_getBuilderState(self):
        return self.slave.state

    def api_getBuildProjectName(self):
        if self.slave.currentBuild is not None:
            return self.slave.currentBuild.getProjectName()
        else:
            return LexbuilderSlaveServer.REQUEST_ERROR

    def api_getBuildName(self):
        if self.slave.currentBuild is not None:
            return self.slave.currentBuild.getBuildName()
        else:
            return LexbuilderSlaveServer.REQUEST_ERROR

    def api_getBuildResult(self):
        if self.slave.currentBuild is not None:
            return self.slave.currentBuild.getState()
        else:
            return LexbuilderSlaveServer.REQUEST_ERROR

    def api_getBuildStartedAt(self):
        if self.slave.currentBuild is not None:
            if self.slave.currentBuild.startedAt is not None:
                return xmlrpclib.DateTime(self.slave.currentBuild.startedAt)
        return LexbuilderSlaveServer.REQUEST_ERROR

    def api_getBuildFinishedAt(self):
        if self.slave.currentBuild is not None:
            if self.slave.currentBuild.finishedAt is not None:
                return xmlrpclib.DateTime(self.slave.currentBuild.finishedAt)
        return LexbuilderSlaveServer.REQUEST_ERROR

    def api_getBuildErrorCode(self):
        if self.slave.currentBuild is not None:
            if self.slave.currentBuild.errorCode is not None:
                return self.slave.currentBuild.errorCode
        return LexbuilderSlaveServer.REQUEST_ERROR

    def api_shutdown(self):
        if self.slave.state == LexbuilderSlave.STATE_IDLE:
            self.shutdownThread = ShutdownThread(self)
            self.shutdownThread.start()
            return LexbuilderSlaveServer.REQUEST_OKAY
        else:
            return LexbuilderSlaveServer.REQUEST_ERROR

class ProjectBuildProcess(MonitoredProcess):

    def __init__(self, supervisor, projectName, configBranchURL):
        MonitoredProcess.__init__(
            self, supervisor, [projectName, configBranchURL])
        self.projectName = projectName

    def setup(self):
        self.buildName = None

    def getState(self):
        if self.state == MonitoredProcess.STATE_ACTIVE:
            return ProjectBuildStates.PENDING
        elif self.state == MonitoredProcess.STATE_FAILED:
            return ProjectBuildStates.FAILED
        elif self.state == MonitoredProcess.STATE_SUCCESS:
            return ProjectBuildStates.SUCCESS
        else:
            return ProjectBuildStates.UNKNOWN

    def preExecution(self):
        logging.debug(
            "Setting state of slave to %s" % LexbuilderSlave.STATE_BUILDING)
        self.master.state = LexbuilderSlave.STATE_BUILDING

    def postExecution(self):
        logging.debug(
            "Setting state of slave to %s" % LexbuilderSlave.STATE_IDLE)
        self.master.state = LexbuilderSlave.STATE_IDLE

    def getCommandPath(self):
        return config.slave("build_command")

    def getProjectName(self):
        return self.projectName

    def getBuildName(self):
        if self.buildName is not None:
            return self.buildName
        else:
            if self.state != MonitoredProcess.STATE_ACTIVE:
                for l in self.stdoutdata.split("\n"):
                    if l.startswith("START"):
                        self.buildName = l.split(" ")[3].strip()
                        return self.buildName
            else:
                try:
                    lfile = open(config.slave("build_statefile"))
                    projectName, buildName = lfile.read().split(" ", 1)
                except Exception:
                    logging.exception(
                        "Error when trying to determine build name")
                    return LexbuilderSlaveServer.REQUEST_ERROR
                if projectName == self.getProjectName():
                    self.buildName = buildName.strip("\n")
                    return self.buildName
        return LexbuilderSlaveServer.REQUEST_ERROR


class ShutdownThread(MonitoredThread):

    def run(self):
        time.sleep(1)
        self.master.stop()
