# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from django.conf import settings
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.core.exceptions import (
    ImproperlyConfigured,
    MiddlewareNotUsed
)
from django.http import (
    HttpResponseForbidden,
    HttpResponseRedirect
)
from django.utils.http import urlquote


class LoginRequiredMiddleware:
    """
    Middleware for requiring users to be logged in to access the website.
    
    Additionally this middleware requires users to be a member of one of the
    teams listed in GROUPS_ALLOWED_LOGIN if set (with the exception of
    superusers).
    
    LOGIN_REQUIRED must be set to True in your Django settings module for
    this middleware to be active.
    """

    def __init__(self):
        if not getattr(settings, 'LOGIN_REQUIRED', False):
            raise MiddlewareNotUsed()
        self.allowed_groups = getattr(settings, 'GROUPS_ALLOWED_LOGIN', None)

    def process_request(self, request):
        if not hasattr(request, 'user'):
            raise ImproperlyConfigured(
                "The Login Required middleware requires the authentication"
                "  middleware to be installed. Edit your MIDDLEWARE_CLASSES"
                "  setting to insert"
                " 'django.contrib.auth.middleware.AuthenticationMiddleware'"
                " before the LoginRequiredMiddleware class.")
        if not request.user.is_authenticated():
            for exempt_url in settings.LOGIN_EXEMPT_URLS:
                if request.path.startswith(exempt_url):
                    return None
            path = urlquote(request.get_full_path())
            tup = settings.LOGIN_URL, REDIRECT_FIELD_NAME, path
            return HttpResponseRedirect('%s?%s=%s' % tup)
        elif self.allowed_groups and not request.user.groups.filter(
                name__in=self.allowed_groups).exists():
            if not request.user.is_superuser:
                return HttpResponseForbidden(
                    "I'm sorry. You're not authorized to access this website.")
        else:
            pass
