# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

from django.contrib import admin

from offspring.web.queuemanager.forms import ProjectBaseForm
from offspring.web.queuemanager.models import (
    BuildRequest,
    BuildResult,
    DailyBuildOrder,
    LaunchpadProject,
    LaunchpadProjectMilestone,
    Lexbuilder,
    Project,
    ProjectGroup,
    ProjectNotificationSubscription,
    Release
)


class LexbuilderAdmin(admin.ModelAdmin):
    list_display = ('name', 'machine_type', 'is_active', 'is_okay', 'current_state', 'current_build', 'updated_at')
    list_filter = ['current_state', 'machine_type', 'is_active', 'is_okay']
    actions=['make_enabled', 'make_disabled']

    def make_disabled(self, request, queryset):
        rows_updated = queryset.update(is_active=False)
        if rows_updated == 1:
            message_bit = "1 lexbuilder was"
        else:
            message_bit = "%s lexbuilders were" % rows_updated
        self.message_user(request, "%s successfully disabled." % message_bit)
    make_disabled.short_description = "Mark selected builders as disabled"

    def make_enabled(self, request, queryset):
        rows_updated = queryset.update(is_active=True)
        if rows_updated == 1:
            message_bit = "1 lexbuilder was"
        else:
            message_bit = "%s lexbuilders were" % rows_updated
        self.message_user(request, "%s successfully enabled." % message_bit)
    make_enabled.short_description = "Mark selected builders as enabled"


class ProjectAdmin(admin.ModelAdmin):
    fields = ['title', 'name', 'arch', 'project_group', 'launchpad_project', 'suite', 'series', 'priority', 'status', 'is_active', 'config_url', 'notes']
    list_display = ('display_name', 'arch', 'series', 'project_group', 'launchpad_project', 'is_active', 'status', 'priority', 'config_url')
    list_filter = ['arch', 'series', 'is_active', 'project_group', 'status']
    search_fields = ['title', 'name', 'arch', 'notes']
    actions = ['raise_priority', 'drop_priority', 'make_disabled', 'make_enabled']
    form = ProjectBaseForm

    def raise_priority(self, request, queryset):
        for project in queryset:
            project.priority += 10
            project.save()
        self.message_user(request, "Priority of selected projects increased by 10.")
    raise_priority.short_description = "Bump up priority of selected projects by 10"

    def drop_priority(self, request, queryset):
        for project in queryset:
            project.priority -= 10
            project.save()
        self.message_user(request, "Priority of selected projects lowered by 10.")
    drop_priority.short_description = "Bump down priority of selected projects by 10"

    def make_disabled(self, request, queryset):
        rows_updated = queryset.update(is_active=False)
        if rows_updated == 1:
            message_bit = "1 project was"
        else:
            message_bit = "%s projects were" % rows_updated
        self.message_user(request, "%s successfully deactivated." % message_bit)
    make_disabled.short_description = "Mark selected projects as unactive"

    def make_enabled(self, request, queryset):
        rows_updated = queryset.update(is_active=True)
        if rows_updated == 1:
            message_bit = "1 project was"
        else:
            message_bit = "%s projects were" % rows_updated
        self.message_user(request, "%s successfully activated." % message_bit)
    make_enabled.short_description = "Mark selected projects as active"


class BuildResultAdmin(admin.ModelAdmin):
    list_display = ('project', 'name', 'builder', 'result')
    list_filter = ['project', 'builder', 'result']
    date_hierarchy = 'finished_at'


class DailyBuildOrderAdmin(admin.ModelAdmin):
    list_display = ('name', 'hour')
    list_filter = ['hour']


class ReleaseAdmin(admin.ModelAdmin):
    list_display = ('__unicode__', 'creator', 'created_at', 'status', 'published_at')
    list_filter = ['status', 'tag']
    search_fields = ['name', 'notes', 'checklist_url']
    date_hierarchy = 'published_at'

    def formfield_for_foreignkey(self, db_field, request, **kwargs):
        if db_field.name == "build":
            kwargs["queryset"] = BuildResult.objects.filter(result="COMPLETED")
            return db_field.formfield(**kwargs)
        return super(ReleaseAdmin, self).formfield_for_foreignkey(db_field, request, **kwargs)


admin.site.register(Lexbuilder, LexbuilderAdmin)
admin.site.register(Project, ProjectAdmin)
admin.site.register(BuildResult, BuildResultAdmin)
admin.site.register(ProjectGroup)
admin.site.register(BuildRequest)
admin.site.register(DailyBuildOrder,DailyBuildOrderAdmin)
admin.site.register(LaunchpadProject)
admin.site.register(LaunchpadProjectMilestone)
admin.site.register(ProjectNotificationSubscription)
admin.site.register(Release, ReleaseAdmin)
