# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__all__ = ["LaunchpadProjectMilestoneHandler", "BuildRequestHandler", \
    "ProjectNotificationSubscriptionHandler", "BuildResultHandler", \
    "BuildManifestHandler", "BuildManifestComparisonHandler", "ReleaseHandler"]

import glob

from django.conf import settings
from django.contrib.auth.decorators import login_required
from django.utils.decorators import method_decorator

from piston.handler import BaseHandler
from piston.utils import (
    rc,
    require_extended
)

from offspring.artifacts import BuildManifest
from offspring.web.queuemanager.models import (
    BuildRequest,
    BuildResult,
    LaunchpadProjectMilestone,
    Project,
    ProjectNotificationSubscription,
    Release
)


class LaunchpadProjectMilestoneHandler(BaseHandler):
    allowed_methods = ('GET')
    model = LaunchpadProjectMilestone
    fields = ['id', 'name', 'description', 'dateTargeted', 'active', 'launchpad_project']
    exclude = ["date_targeted"]

    def read(self, request):
        milestones = LaunchpadProjectMilestone.objects.exclude(date_targeted__exact=None)
        return milestones


class BuildManifestHandler(BaseHandler):
    allowed_methods = ('GET')

    def read(self, request, projectName, buildName):
        try:
            project = Project.objects.get(pk=projectName)
            build = BuildResult.objects.get(project=project, name=buildName)
        except: 
            return rc.NOT_FOUND
        manifestPath = glob.glob(build.result_directory + '/images/*/*.manifest')[0]
        return [ {'package' : package, 'version' : version} for package, version in BuildManifest(manifestPath).packages.iteritems() ]


class BuildManifestComparisonHandler(BaseHandler):
    allowed_methods = ('GET')

    def read(self, request, projectName, buildName):
        try:
            project = Project.objects.get(pk=projectName)
            base_build = BuildResult.objects.get(project=project, name=buildName)
        except:
            return rc.NOT_FOUND
        if request.GET.has_key('build'):
            try:
                target_build = BuildResult.objects.get(project=project, name=request.GET['build'])
            except:
                return rc.NOT_FOUND
        else:
            return rc.BAD_REQUEST
        try:
            base_manifestPath = glob.glob(base_build.result_directory + '/images/*/*.manifest')[0]
            target_manifestPath = glob.glob(target_build.result_directory + '/images/*/*.manifest')[0]
        except:
            return rc.NOT_HERE
        base_manifest = BuildManifest(base_manifestPath)
        target_manifest = BuildManifest(target_manifestPath)
        output = []
        for changeType, packages in base_manifest.compareTo(target_manifest).iteritems():
            output += [ { 'changeType' : changeType, 'package' : package, 'base_version' : base_manifest.packages.get(package, None), 'target_version' : target_manifest.packages.get(package, None) } for package in packages ]
        return output


class ProjectNotificationSubscriptionHandler(BaseHandler):
    allowed_methods = ('GET', 'POST', 'DELETE')
    model = ProjectNotificationSubscription
    fields = ['id']

    def read(self, request, projectName):
        try:
            p = Project.objects.get(pk=projectName)
            s = ProjectNotificationSubscription.objects.get(project=p, user=request.user)
        except:
            return rc.NOT_FOUND
        return s

    @method_decorator(login_required)
    def delete(self, request, projectName):
        p = Project.objects.get(pk=projectName)
        s = ProjectNotificationSubscription.objects.filter(project=p, user=request.user)
        try:
            s.delete()
        except:
            return rc.BAD_REQUEST
        return s

    @method_decorator(login_required)
    def create(self, request, projectName):
        try:
            p = Project.objects.get(pk=projectName)
        except:
            return rc.NOT_HERE
        try:
            s = ProjectNotificationSubscription.objects.get(project=p, user=request.user)
            if s is not None:
                return rc.DUPLICATE_ENTRY
        except:
            pass
        try:
            s = ProjectNotificationSubscription(project=p, user=request.user)
            s.save()
        except:
            return rc.BAD_REQUEST
        return s


class BuildResultHandler(BaseHandler):
    allowed_methds = ('GET')
    model = BuildResult
    fields = ( 
        ('builder', 
            ('name','updated_at','is_active', 'created_at', 'current_state', 'previous_state', 'is_okay', 'machine_type')),
        ('requestor',
            ('username', 'first_name', 'last_name', 'is_active', 'email')),
        'name', 'project', 'requested_at', 'dispatched_at', 'started_at', 'finished_at', 'reason', 'result', 'notes', 'is_released')

    def read(self, request, projectName=None, buildName=None, builderName=None):
        results = BuildResult.objects.all()
        if projectName is not None:
            results = results.filter(project__name=projectName)
        if buildName is not None:
            results = results.filter(name=buildName)
        if builderName is not None:
            results = results.filter(builder__name=builderName)
        return results


class BuildRequestHandler(BaseHandler):
    allowed_methods = ('GET', 'POST', 'DELETE')
    model = BuildRequest
    fields = ['id', 'project', 'requestor', 'created_at', 'score', 'reason']

    def read(self, request, projectName=None, request_id=None):
        try:
            if projectName is not None:
                requests = BuildRequest.objects.filter(project__name=projectName)
            else:
                requests = BuildRequest.objects
        except:
            return rc.NOT_FOUND
        if request_id is not None:
            return requests.get(pk=request_id)
        else:
            return requests

    @method_decorator(login_required)
    def delete(self, request, projectName=None, request_id=None):
        if request.user.has_perm("queuemanager.can_build"):
            try:
                if projectName is not None:
                    requests = BuildRequest.objects.filter(project__name=projectName)
                else:
                    requests = BuildRequest.objects
            except:
                return RC.NOT_FOUND
            if request_id is not None:
                try:
                    requests.get(pk=request_id).delete()
                    return rc.DELETED
                except:
                    return rc.NOT_FOUND
            else:
                try:
                    requests.delete()
                    return rc.DELETED
                except:
                    return rc.NOT_FOUND
        else:
            return rc.FORBIDDEN

    @method_decorator(login_required)
    def create(self, request, projectName):
        if request.user.has_perm("queuemanager.can_build"):
            try:
                p = Project.objects.get(pk=projectName)
                if request.POST.has_key('build_reason'):
                    request = BuildRequest.queue_build(p, request.user, reason=request.POST['build_reason'], scoreBonus=25)
                else:
                    request = BuildRequest.queue_build(p, request.user, scoreBonus=25)
                request.save()
                return rc.CREATED
            except:
                return rc.BAD_REQUEST
        else:
            return rc.FORBIDDEN


class ReleaseHandler(BaseHandler):
    allowed_methods = ('GET')
    model = Release
    fields = ('name', 'build', 'milestone', 'tag',
        ('creator',
            ('username', 'first_name', 'last_name', 'is_active', 'email')),
        'created_at', 'published_at', 'status', 'checklist_url', 'notes')

    def read(self, request, projectName):
        try:
            releases = Release.objects.filter(build__project__name=projectName)
        except:
            return rc.NOT_FOUND
        return releases
