# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

__all__ = ["Project", "ProjectGroup", "Lexbuilder", "BuildRequest", \
    "BuildResult", "DailyBuildOrder", "LaunchpadProject", "LaunchpadProjectMilestone", \
    "ProjectNotificationSubscription", 'Release']

from datetime import (
    date,
    datetime,
    timedelta
)
import math

from django.conf import settings
from django.contrib.auth.models import User
from django.db import (
    connection,
    models
)

from offspring.enums import ProjectBuildStates


ARCH_CHOICES = (
    (u'i386', u'i386'),
    (u'amd64', u'amd64'),
    (u'lpia', u'lpia'),
    (u'armel', u'armel'),
)

HOURS_CHOICES = (
    (0,  '0000 (12am)'),
    (1,  '0100 (1am)'),
    (2,  '0200 (2am)'),
    (3,  '0300 (3am)'),
    (4,  '0400 (4am)'),
    (5,  '0500 (5am)'),
    (6,  '0600 (6am)'),
    (7,  '0700 (7am)'),
    (8,  '0800 (8am)'),
    (9,  '0900 (9am)'),
    (10, '1000 (10am)'),
    (11, '1100 (11am)'),
    (12, '1200 (12pm)'),
    (13, '1300 (1pm)'),
    (14, '1400 (2pm)'),
    (15, '1500 (3pm)'),
    (16, '1600 (4pm)'),
    (17, '1700 (5pm)'),
    (18, '1800 (6pm)'),
    (19, '1900 (7pm)'),
    (20, '2000 (8pm)'),
    (21, '2100 (9pm)'),
    (22, '2200 (10pm)'),
    (23, '2300 (11pm)'),
)


class ProjectGroup(models.Model):
    name = models.CharField(max_length=200, primary_key=True, unique=True)
    title = models.CharField(max_length=200)

    class Meta:
        db_table = "projectgroups"

    def __unicode__(self):
        return self.title

    def display_name(self):
        return "%s (%s)" % (self.title, self.name)

    def get_absolute_url(self):
        return "/project-groups/%s/" % self.name

    @property
    def projects(self):
        return Project.objects.filter(project_group=self)

    @property
    def builder(self): 
        return None

    @property
    def is_active(self):
        return bool(Project.objects.filter(project_group=self).exclude(is_active=False).count() != 0)

    @property
    def needs_build(self):
        return BuildRequest.objects.filter(project__project_group=self).exists()

    @property
    def average_build_time(self):
        cursor = connection.cursor()
        cursor.execute("SELECT SUM(finished_at - started_at) / count(*) from buildresults br, projects WHERE br.project_name = projects.name AND projects.project_group_id = %s AND br.result = %s", [self.name, "COMPLETED"])
        row = cursor.fetchone()
        if row[0] is not None:
            result = str(row[0])
            hours, minutes, seconds = result.split(":")
            #XXX: The following can't handle builds that have an average 
            #XXX: duration of greater then 23 hours and 59 minutes and 59 
            #XXX: seconds (e.g. if a build gets stuck and lasts over a day).
            try:
                if int(hours) < 1:
                    return "%s mins, %s secs" % (minutes, math.floor(float(seconds)))
                else:
                    return "%s hrs, %s mins, %s secs" % (hours, minutes, math.floor(float(seconds)))
            except ValueError:
                #XXX: We'll potentially incorrectly report >1 day until we fix above.
                return ">1 Day"
        return "UNKNOWN"

    def latest_build(self):
        try:
            return BuildResult.objects.filter(project__project_group = self).exclude(result = None).exclude(started_at = None).latest()
        except self.model.DoesNotExist:
            return None

    def latest_build_name(self):
        try:
            return self.latest_build().name
        except self.model.DoesNotExist:
            return u""

    def latest_build_date(self):
        try:
            return self.latest_build().started_at
        except self.model.DoesNotExist:
            return u""


class Project(models.Model):
    #XXX: This should probably be managed in the database.
    STATUS_CHOICES = (
        (u'devel', u'Active Development'),
        (u'frozen', u'Pre-release Freeze'),
        (u'supported', u'Supported'),
        (u'suspended', u'Suspended'),
        (u'experimental', u'Experimental'),
        (u'obsolete', u'Obsolete'),
    )

    name = models.SlugField('codename', max_length=200, primary_key=True, unique=True)
    title = models.CharField('project title', max_length=30)
    project_group = models.ForeignKey(ProjectGroup, blank=True, null=True)
    priority = models.IntegerField(default=50)
    is_active = models.BooleanField(default=True)
    arch = models.CharField('architecture', max_length=10, choices=ARCH_CHOICES)
    suite = models.CharField('project archive', max_length=200, blank=True)
    series = models.CharField('ubuntu series', max_length=200, default="lucid")
    launchpad_project = models.ForeignKey("LaunchpadProject", blank=True, null=True)
    status = models.CharField('status', max_length=200, null=True, choices=STATUS_CHOICES)
    config_url = models.CharField('config URL', max_length=200) # Add some kind of validator here.
    notes = models.TextField(blank=True, null=True)

    class Meta:
        db_table="projects"
        permissions = (
            ("can_build", "Can mark a project for building"),
        )

    def __unicode__(self):
        return self.display_name()

    def get_absolute_url(self):
        return "/projects/%s/" % self.name

    def display_name(self):
        return "%s (%s)" % (self.title, self.name)

    def builder(self): 
        return Lexbuilder.objects.filter(current_job__project = self)[0:1].get()

    @property
    def needs_build(self):
        return BuildRequest.objects.filter(project=self.name).exists()

    @property
    def average_build_time(self):
        cursor = connection.cursor()
        cursor.execute("SELECT SUM(finished_at - started_at) / count(*) from buildresults WHERE project_name = %s AND result = %s", [self.name, "COMPLETED"])
        row = cursor.fetchone()
        if row[0] is not None:
            result = str(row[0])
            hours, minutes, seconds = result.split(":")
            #XXX: The following can't handle builds that have an average 
            #XXX: duration of greater then 23 hours and 59 minutes and 59 
            #XXX: seconds (e.g. if a build gets stuck and lasts over a day).
            try:
                if int(hours) < 1:
                    return "%s mins, %s secs" % (minutes, math.floor(float(seconds)))
                else:
                    return "%s hrs, %s mins, %s secs" % (hours, minutes, math.floor(float(seconds)))
            except ValueError:
                #XXX: We'll potentially incorrectly report >1 day until we fix above.
                return ">1 Day"
        return "UNKNOWN"

    def latest_build(self):
        try:
            return BuildResult.objects.filter(project = self.name).exclude(result = None).exclude(started_at = None).latest()
        except self.model.DoesNotExist:
            return None

    def latest_build_name(self):
        try:
            return self.latest_build().name
        except self.model.DoesNotExist:
            return u""

    def latest_build_date(self):
        try:
            return self.latest_build().started_at
        except self.model.DoesNotExist:
            return u""


class Lexbuilder(models.Model):
    name = models.SlugField(max_length=200, unique=True)
    uri = models.CharField(max_length=200)
    current_state = models.CharField(max_length=200, default="UNKNOWN", editable=False)
    previous_state = models.CharField(max_length=200, default="UNKNOWN", editable=False)
    created_at = models.DateTimeField('date created', auto_now_add=True)
    updated_at = models.DateTimeField('date updated', auto_now=True)
    is_active = models.BooleanField(default=True)
    is_okay = models.BooleanField(default=True, editable=False)
    machine_type = models.CharField(max_length=200, default="x86_64")
    current_job = models.ForeignKey("BuildResult", db_column="current_job_id", editable=False, blank=True, null=True)
    notes = models.TextField('whiteboard', blank=True, null=True)

    class Meta:
        db_table = "lexbuilders"

    def __unicode__(self):
        return self.name

    @property
    def status(self):
        if self.is_active:
            if self.is_okay:
                return self.current_state
            else:
                return "PROBLEM"
        else:
            return "OFFLINE"

    @property
    def current_build_projectname(self):
        return self.current_job.project.name

    @property
    def current_build_buildname(self):
        return self.current_job.name

    @property
    def current_build(self):
        return self.current_job

    @property
    def estimated_completion(self):
        if self.current_job is not None:
            #XXX: Duplication of code in Project to get average build time
            cursor = connection.cursor()
            cursor.execute("SELECT SUM(finished_at - started_at) / count(*) from buildresults WHERE project_name = %s AND result = %s", [self.current_job.project.name, "COMPLETED"])
            row = cursor.fetchone()
            if row[0] is not None:
                averageBuildTime = row[0]
                currentElapsedTime = datetime.now() - self.current_job.started_at
                estimatedTimeRemaining = averageBuildTime - currentElapsedTime
                #XXX: Duplication of code in Project to format output
                result = str(estimatedTimeRemaining)
                if result.startswith("-"):
                    return "ASAP"
                else:
                    hours, minutes, seconds = result.split(":")
                    if int(hours) < 1:
                        return "%s mins, %s secs" % (minutes, math.floor(float(seconds)))
                    else:
                        return "%s hrs, %s mins, %s secs" % (hours, minutes, math.floor(float(seconds)))
            else:
                return "Unknown"
        else:
            return "No build in progress"

    @property
    def average_load(self):
        cursor = connection.cursor()
        cursor.execute("SELECT EXTRACT(EPOCH FROM sum(finished_at - started_at) / COUNT(DISTINCT date(started_at))) / EXTRACT(EPOCH FROM INTERVAL '1 day') * 100 FROM buildresults WHERE builder_id='%s' AND date(started_at) >= date(LOCALTIMESTAMP - interval '14 days') AND date(started_at) != date(LOCALTIMESTAMP);", [self.id])
        return cursor.fetchone()[0]


class BuildResult(models.Model):
    name = models.CharField('build name', max_length=200, blank=True, null=True, editable=False)
    project = models.ForeignKey(Project, db_column="project_name", editable=False)
    builder = models.ForeignKey(Lexbuilder, blank=True, null=True, editable=False)
    started_at = models.DateTimeField('date started', editable=False, blank=True, null=True)
    finished_at = models.DateTimeField('date finished', editable=False, blank=True, null=True)
    result = models.CharField(max_length=200, editable=False, null=True)
    requestor = models.ForeignKey(User, db_column="requestor_id", blank=True, null=True, editable=False)
    reason = models.CharField('request reason', blank=True, null=True, max_length=200, editable=False)
    requested_at = models.DateTimeField('date created', auto_now_add=True, blank=True, null=True, editable=False)
    dispatched_at = models.DateTimeField('date dispatched', editable=False, null=True, blank=True) 
    notes = models.CharField(max_length=200, null=True, blank=True)

    class Meta:
        db_table = "buildresults"
        get_latest_by = "started_at"

    def __unicode__(self):
        return u"%s %s" % (self.project.name, self.name)

    @property
    def is_released(self):
        return bool(Release.objects.filter(build=self).count() != 0)

    @property
    def queue_duration(self):
        if self.requested_at is not None and self.dispatched_at is not None:
            return self.dispatched_at - self.requested_at

    @property
    def build_duration(self):
        if self.started_at is not None and self.finished_at is not None:
            return self.finished_at - self.started_at

    @property
    def result_directory(self):
        if self.name:
            try:
                base_dir = getattr(settings, 'BUILDRESULTS_DIRECTORY', '/srv/builds/')
                buildDate, buildId = self.name.rsplit('-', 1)
            except:
                return None
            return "%s/%s/%s/%s" % (base_dir, self.project.name, buildDate, buildId)
        else:
            return None

    def recentBuildFailures(limit=5):
        #XXX: Should be method of custom model manager.
        threshold = datetime.now() - timedelta(weeks=2)
        r = BuildResult.objects.select_related(
            'project').filter(finished_at__gte = threshold).filter(
            result = ProjectBuildStates.FAILED).order_by(
            "finished_at").reverse()
        if limit is None:
            return r
        else:
            return r[:limit]
    recentBuildFailures = staticmethod(recentBuildFailures)


class BuildRequest(models.Model):
    project = models.ForeignKey(Project, db_column="project_name")
    requestor = models.ForeignKey(User, db_column="requestor_id", blank=True, null=True)
    created_at = models.DateTimeField('date created', auto_now_add=True)
    score = models.IntegerField(default=10)
    reason = models.TextField('request reason', blank=True, max_length=200)

    class Meta:
        db_table = "buildrequests"

    def __unicode__(self):
        if self.requestor:
            requestor_name = self.requestor.username
        else:
            requestor_name = "Offspring Bot"
        if len(self.reason):
            comment = " (%s)" % self.reason
        else:
            comment = ""
        return u"%s requested by %s%s." % (self.project, requestor_name, comment)

    def queue_build(project, requestor, reason="", scoreBonus=0):
        calculatedScore = project.priority + scoreBonus
        if project.is_active:
            calculatedScore += 10
        if bool(requestor.groups.filter(name__exact="OEM Solutions Group").values('name')):
            calculatedScore += 50
        if bool(requestor.groups.filter(name__exact="OEM Solutions Release Engineers").values('name')):
            calculatedScore += 50
        # Easter Egg :) - +10 for giving a reason, +100 if it sounds dire ;]
        if reason != "":
            calculatedScore += 10
            if set(reason.lower().split(" ")).intersection(set(['urgent', 'fire', 'critical', 'drill', 'omgz'])):
                calculatedScore += 100
        return BuildRequest(project=project, requestor=requestor, reason=reason, score=calculatedScore)
    queue_build = staticmethod(queue_build)


class DailyBuildOrder(models.Model):
    name = models.CharField('daily build order', max_length=200)
    hour = models.PositiveIntegerField('start time', choices=HOURS_CHOICES)
    projects = models.ManyToManyField(Project)

    class Meta:
        db_table = "dailybuildorders"

    def __unicode__(self):
        return self.name


class LaunchpadProject(models.Model):
    name = models.CharField('launchpad project name', max_length=200, primary_key=True, unique=True)
    title = models.CharField('launchpad project title', max_length=200, null=True, blank=True, editable=False)

    class Meta:
        db_table = "launchpad_projects"

    def __unicode__(self):
        return self.name


class LaunchpadProjectMilestone(models.Model):
    name = models.CharField('milestone name', max_length=200, editable=False)
    launchpad_project = models.ForeignKey(LaunchpadProject, blank=True, null=True, editable=False)
    title = models.CharField('milestone title', max_length=200, blank=True, editable=False)
    date_targeted = models.DateField('date targeted', blank=True, null=True, default=None, editable=False)
    active = models.BooleanField(default=True, editable=False)

    class Meta:
        db_table = "launchpad_project_milestones"

    def __unicode__(self):
        if len(self.title) > 0:
            return "%s %s '%s'" % (self.launchpad_project, self.name, self.title)
        else:
            return "%s %s" % (self.launchpad_project, self.name)

    def upcoming_milestones(limit=3):
        #XXX: Should be method of custom model manager.
        r = LaunchpadProjectMilestone.objects.filter(
            date_targeted__gte=date.today()
        ).order_by("date_targeted")
        if limit is None:
            return r
        else:
            return r[:limit]
    upcoming_milestones = staticmethod(upcoming_milestones)

    def recent_milestones(limit=3):
        #XXX: Should be method of custom model manager.
        r = LaunchpadProjectMilestone.objects.filter(
            date_targeted__lte=date.today()
        ).order_by("date_targeted").reverse()
        if limit is None:
            return r
        else:
            return r[:limit]
    recent_milestones = staticmethod(recent_milestones)

    @property
    def dateTargeted(self):
        return self.date_targeted

    @property
    def description(self):
        if self.title is not None and len(self.title) > 0:
            return self.title
        else:
            return self.name

    def get_url(self):
        return "http://launchpad.net/%s/+milestone/%s" % (self.launchpad_project, self.name)


class ProjectNotificationSubscription(models.Model):
    user = models.ForeignKey(User, db_column="user_id")
    project = models.ForeignKey(Project, db_column="project_name")

    class Meta:
        db_table = "project_notification_subscriptions"

    def __unicode__(self):
        return "%s subscribed to %s" % (self.user, self.project)


class Release(models.Model):
    STATUS_PENDING, STATUS_PUBLISHED, STATUS_REMOVED, STATUS_MISSING, STATUS_ERROR = range(5)
    STATUS_CHOICES = (
        (STATUS_PENDING, 'Pending Publication'),
        (STATUS_PUBLISHED, 'Published'),
        (STATUS_REMOVED, 'Deleted'),
        (STATUS_MISSING, 'Publication Missing'),
        (STATUS_ERROR, 'Error Publishing'),
    )

    #XXX: This should probably be managed in the database.
    TAG_CHOICES = (
        ("T", "Test Release"),
        ("M", "Milestone"),
        ("RC", "Release Candidate"),
        ("GM", "Gold Master"),
    )

    build = models.OneToOneField(BuildResult, primary_key=True, unique=True)
    name = models.CharField('release name', max_length=200)
    milestone = models.ForeignKey(LaunchpadProjectMilestone, null=True, blank=True)
    tag = models.CharField(max_length=5, null=True, blank=True, choices=TAG_CHOICES)
    creator = models.ForeignKey(User)
    created_at = models.DateTimeField('date created', auto_now_add=True)
    updated_at = models.DateTimeField('date updated', auto_now=True)
    published_at = models.DateTimeField('date published', null=True, editable=False)
    checklist_url = models.URLField('checklist URL', verify_exists=False, blank=True)
    notes = models.TextField('release notes', blank=True)
    status = models.PositiveSmallIntegerField(choices=STATUS_CHOICES, default=STATUS_PENDING)

    class Meta:
        db_table = "releases"

    def __unicode__(self):
        return "%s release '%s' (%s)" % (
            self.build.project.name, self.name, self.build.name)
