# Copyright 2010 Canonical Ltd.  This software is licensed under the
# GNU Affero General Public License version 3 (see the file LICENSE).

import urlparse

from django.contrib.auth.decorators import (
    login_required, 
    permission_required
)
from django.core.paginator import (
    Paginator,
    InvalidPage,
    EmptyPage
)
from django.db.models import Q
from django.http import (
    HttpResponse,
    HttpResponseRedirect,
    HttpResponseForbidden
)
from django.middleware import csrf
from django.shortcuts import (
    render_to_response,
    get_object_or_404
)
from django.template import (
    Context,
    RequestContext,
    Template
)
from django.utils.html import escape
from django.views.generic.create_update import (
    create_object,
    update_object
)
from django.views.generic.list_detail import (
    object_list,
    object_detail
)

from pygooglechart import PieChart3D

from offspring import config
from offspring.enums import ProjectBuildStates
from offspring.web.queuemanager.forms import (
    CreateProjectForm,
    EditProjectForm,
    LaunchpadProjectForm,
    ReleaseForm,
)
from offspring.web.queuemanager.models import (
    BuildRequest,
    BuildResult,
    LaunchpadProjectMilestone,
    Project,
    ProjectGroup,
    Release
)

#XXX: Reduce horrible duplication that occurs in these functions (mostly done).
#TODO: Thanks to reduced duplication, lets start using more generic views.

@login_required
def secure_object_list(request, **page_info):
    '''Wrap object_list in login_required decorator.'''
    return object_list(request, **page_info)

@login_required
def secure_object_detail(request, **page_info):
    '''Wrap object_detail in login_required decorator.'''
    return object_detail(request, **page_info)

def projects(request):
    projects = Project.objects.select_related('project_group').order_by("project_group")
    if request.path == "/" or request.path == "":
        pillar = "home"
        #XXX: Move logic for determining active projects into model manager.
        projects = projects.exclude(Q(is_active=False) & Q(dailybuildorder=None))
    else:
        pillar = "projects"
    pageData = { 
        'projects' : projects,
        'pillar' : pillar
    }
    return render_to_response(
        'queuemanager/projects.html', pageData,
        context_instance=RequestContext(request))

def builds(request, projectName=None):
    if projectName is not None:
        p = get_object_or_404(Project, pk=projectName)
        buildResults_list = BuildResult.objects.filter(
            project = projectName).order_by("finished_at").reverse()
    else:
        p = None
        buildResults_list = BuildResult.objects.select_related(
            'builder', 'requestor', 'project').order_by("finished_at").reverse()
    paginator = Paginator(buildResults_list, 15)

    try:
        page = int(request.GET.get('page', '1'))
    except ValueError:
        page = 1

    try:
        buildResults = paginator.page(page)
    except (EmptyPage, InvalidPage):
        buildResults = paginator.page(paginator.num_pages)

    pageData = { 
        'project' : p, 
        'build_results' : buildResults,
        'pillar' : 'builds' 
    }
    return render_to_response(
        'queuemanager/builds.html', pageData, 
        context_instance=RequestContext(request))


def build_details(request, projectName, buildName):
    b = get_object_or_404(
        BuildResult.objects.select_related('builder', 'project', 'requestor'),
        project__pk=projectName, name=buildName)
    pageData = { 
        'pillar' : 'builds',
        'upcoming_milestones' : LaunchpadProjectMilestone.upcoming_milestones(),
        'recent_milestones' : LaunchpadProjectMilestone.recent_milestones(),
        'project' : b.project, 
        'build' : b
    }
    return render_to_response(
        'queuemanager/build_details.html', pageData, 
        context_instance=RequestContext(request))

@login_required
def build_publish(request, projectName, buildName):
    p = get_object_or_404(Project, pk=projectName)
    b = get_object_or_404(
        BuildResult, project=p, name=buildName, 
        result=ProjectBuildStates.SUCCESS, release=None)
    if request.user.has_perm('queuemanager.add_release'):
        r = Release()
        r.build = b
        r.creator = request.user
        if request.method == 'POST':
            f = ReleaseForm(request.POST, instance=r)
            if f.is_valid():
                f.save()
                return HttpResponseRedirect(p.get_absolute_url())
        else:
            f = ReleaseForm(instance=r)
        f.fields['milestone'].queryset = LaunchpadProjectMilestone.objects.filter( \
            launchpad_project=f.instance.build.project.launchpad_project \
            ).exclude(active=False)
        pageData = { 
            'pillar' : 'builds',
            'project' : p,
            'build' : b,
            'form' : f 
        }
        return render_to_response(
            'queuemanager/build_publish.html', pageData, 
            context_instance=RequestContext(request))
    else:
        return HttpResponseForbidden(render_to_response('403.html'))

def project_create(request):
    return create_object(
        request,
        form_class=CreateProjectForm, 
        login_required=True,
        template_name='queuemanager/project_create.html',
        extra_context={ 'pillar' : 'projects' },
    )
project_create = permission_required('queuemanager.add_project')(project_create)

def project_edit(request, projectName):
    return update_object(
        request,
        form_class=EditProjectForm,
        login_required=True,
        template_name='queuemanager/project_edit.html',
        template_object_name='project',
        extra_context={ 'pillar' : 'projects' },
        object_id=projectName,
    )
project_edit = permission_required('queuemanager.change_project')(project_edit)

def project_details(request, projectName):
    p = get_object_or_404(
        Project.objects.select_related('project_group', 'launchpad_project'), 
        pk=projectName)
    project_build_results = BuildResult.objects.filter(
        project = p).exclude(result = None)
    build_stats = { 
        'fail_count' : project_build_results.filter(result = ProjectBuildStates.FAILED).count(),
        'success_count' : project_build_results.filter(result = ProjectBuildStates.SUCCESS).count() 
    }

    if len(project_build_results):
        chart = PieChart3D(425, 100)
        chart.add_data([build_stats['fail_count'], build_stats['success_count']])
        chart.set_colours(['D01B1B', '60B33C'])
        chart.set_pie_labels(["Failed Builds", "Successful Builds"])
        build_stats['pie_chart'] = chart.get_url()

    releases_uri_base = config.web('releases_uri_base')
    if not releases_uri_base.endswith('/'):
        releases_uri_base += '/'
    releases_url = urlparse.urljoin(releases_uri_base, p.name + '/images/')

    pageData = {
        'build_stats' : build_stats,
        'csrf_token' : csrf.get_token(request),
        'include_sources_list': config.web('archive_uri', None) is not None,
        'pillar' : 'projects',
        'project' : p,
        'project_build_results' : project_build_results,
        'releases_url': releases_url,
        }
    return render_to_response(
        'queuemanager/project_details.html', pageData, 
        context_instance=RequestContext(request))

def projectgroup_details(request, projectGroupName):
    pg = get_object_or_404(
        ProjectGroup, pk=projectGroupName)
    projectgroup_build_results = BuildResult.objects.filter(
        project__project_group = pg).exclude(result = None)
    build_stats = { 
        'fail_count' : projectgroup_build_results.filter(result = ProjectBuildStates.FAILED).count(),
        'success_count' : projectgroup_build_results.filter(result = ProjectBuildStates.SUCCESS).count()
    }

    if len(projectgroup_build_results):
        chart = PieChart3D(425, 150)
        chart.add_data([build_stats['fail_count'], build_stats['success_count']])
        chart.set_colours(['D01B1B', '60B33C'])
        chart.set_pie_labels(["Failed Builds", "Successful Builds"])
        build_stats['pie_chart'] = chart.get_url()

    pageData = { 
        'projectGroup' : pg, 
        'pillar' : 'projects', 
        'build_stats' : build_stats,
        'projectgroup_build_results' : projectgroup_build_results,
        'pillar' : 'projects'
    }
    return render_to_response(
        'queuemanager/projectgroup_details.html', pageData, 
        context_instance=RequestContext(request))

@login_required
def project_development_sourcesList(request, projectName):
    c = Context({"projectName": projectName})
    if request.GET.get('action') == "view":
        t = Template(
            "{% load lexbuilder_helpers %}<link rel=\"stylesheet\" type=\"text/css\" href=\"/assets/css/project_details.css\"/><div id=\"sourcesListModalContent\"><pre>{{ projectName|generate_devel_sourceslist }}</pre></div>")
        return HttpResponse(t.render(c))
    else:
        t = Template(
            "{% load lexbuilder_helpers %}{{ projectName|generate_devel_sourceslist }}")
        return HttpResponse(
            t.render(c), content_type="text/x-apt-sources-list")

@login_required
def queue_build(request, projectName):
    p = get_object_or_404(Project, pk=projectName)
    if p.needs_build:
        BuildRequest.objects.filter(project = p).delete()
    else:
        if request.POST.has_key('build_reason'):
            build_reason = request.POST['build_reason']
        else:
            build_reason = ""
        request = BuildRequest.queue_build(
            p, request.user, build_reason, scoreBonus=25)
        request.save()
    return HttpResponseRedirect(p.get_absolute_url())
queue_build = permission_required('queuemanager.can_build')(queue_build)

def schedule(request):
    pageData = { 
        'pillar' : 'schedule',
        'upcoming_milestones' : LaunchpadProjectMilestone.upcoming_milestones(),
        'recent_milestones' : LaunchpadProjectMilestone.recent_milestones()
    }
    return render_to_response(
        'queuemanager/schedule.html', pageData,
        context_instance=RequestContext(request))

def milestones(request, milestoneType="all"):
    if milestoneType == "future":
        milestones = LaunchpadProjectMilestone.upcoming_milestones(limit=None)
    elif milestoneType == "past":
        milestones = LaunchpadProjectMilestone.recent_milestones(limit=None)
    else:
        milestoneType = "all"
        milestones =  LaunchpadProjectMilestone.objects.all().order_by("date_targeted").reverse()
    paginator = Paginator(milestones, 15)

    try:
        page = int(request.GET.get('page', '1'))
    except ValueError:
        page = 1

    try:
        milestonePage = paginator.page(page)
    except (EmptyPage, InvalidPage):
        milestonePage = paginator.page(paginator.num_pages)

    pageData = {
        'milestones' : milestonePage,
        'milestoneType' : milestoneType,
        'pillar' : 'schedule'
    }
    return render_to_response(
        'queuemanager/milestones.html', pageData, 
        context_instance=RequestContext(request))

def launchpad_project_create(request):
    if request.method == "POST":
        form = LaunchpadProjectForm(request.POST)
        if form.is_valid():
            try:
                newObject = form.save()
            except forms.ValidationError, error:
                newObject = None
            if newObject:
                return HttpResponse('<script type="text/javascript">opener.dismissAddAnotherPopup(window, "%s", "%s");</script>' % \
                    (escape(newObject._get_pk_val()), escape(newObject)))
    else:
        form = LaunchpadProjectForm()
    pageContext = {'form': form}
    return render_to_response(
        'queuemanager/launchpad_project_create_popup.html', 
        pageContext, 
        context_instance=RequestContext(request))
launchpad_project_create = permission_required('queuemanager.add_launchpadproject')(launchpad_project_create)
